@@ -62,7 +62,7 @@ Settings::SettingsData* Settings::SettingsData::instance()
 }
 
 Settings::SettingsData::SettingsData( const QString& imageDirectory )
-    : _hasAskedAboutTimeStamps( false ), _imageDirectory( imageDirectory )
+    : _imageDirectory( imageDirectory )
 {
     QPixmapCache::setCacheLimit( thumbnailCache() * 1024 );
     _smoothScale = value( QString::fromLatin1( "Viewer" ), QString::fromLatin1( "smoothScale" ), true );
@@ -79,34 +79,14 @@ void Settings::SettingsData::setSmoothScale( bool b )
     setValue( QString::fromLatin1( "Viewer" ), QString::fromLatin1( "smoothScale" ), b );
 }
 
-bool Settings::SettingsData::trustTimeStamps()
+void Settings::SettingsData::setThumbnailAspectRatio( Settings::ThumbnailAspectRatio aspect )
 {
-    if ( tTimeStamps() == Always )
-        return true;
-    else if ( tTimeStamps() == Never )
-        return false;
-    else {
-        if (!_hasAskedAboutTimeStamps ) {
-            QApplication::setOverrideCursor( Qt::ArrowCursor );
-            int answer = KMessageBox::questionYesNo( 0, i18n("New images were found. Should I trust their time stamps?"),
-                                                     i18n("Trust Time Stamps?") );
-            QApplication::restoreOverrideCursor();
-            if ( answer == KMessageBox::Yes )
-                _trustTimeStamps = true;
-            else
-                _trustTimeStamps = false;
-            _hasAskedAboutTimeStamps = true;
-        }
-        return _trustTimeStamps;
-    }
+    setValue( STR("Thumbnails"), STR("thumbnailAspectRatio"), (int) aspect);
 }
-void Settings::SettingsData::setTTimeStamps( TimeStampTrust t )
+
+Settings::ThumbnailAspectRatio Settings::SettingsData::thumbnailAspectRatio() const
 {
-    setValue( STR("General"), STR("trustTimeStamps"), (int) t );
-}
-Settings::TimeStampTrust Settings::SettingsData::tTimeStamps() const
-{
-    return (TimeStampTrust) value(  STR("General"), STR("trustTimeStamps"), (int) Always );
+    return (ThumbnailAspectRatio) value(  STR("Thumbnails"), STR("thumbnailAspectRatio"), (int) Aspect_4_3);
 }
 
 void Settings::SettingsData::setViewerStandardSize(StandardViewSize v)
@@ -218,6 +198,7 @@ QString Settings::SettingsData::fileForCategoryImage( const QString& category, Q
 {
     QString dir = imageDirectory() + STR("CategoryImages" );
     member.replace( ' ', '_' );
+    member.replace( '/', '_' );
     QString fileName = dir + STR("/%1-%2.jpg").arg( category ).arg( member );
     return fileName;
 }
@@ -388,7 +369,7 @@ QSize Settings::SettingsData::value( const QString& group, const QString& option
     return config->readSizeEntry( option, &defaultValue );
 }
 
-Set<QString> Settings::SettingsData::value(const QString& group, const QString& option, const Set<QString>& defaultValue ) const
+StringSet Settings::SettingsData::value(const QString& group, const QString& option, const StringSet& defaultValue ) const
 {
     KConfig* config = kapp->config();
     config->setGroup( group );
@@ -432,7 +413,7 @@ void Settings::SettingsData::setValue( const QString& group, const QString& opti
     config->writeEntry( option, value );
 }
 
-void Settings::SettingsData::setValue( const QString& group, const QString& option, const Set<QString>& value )
+void Settings::SettingsData::setValue( const QString& group, const QString& option, const StringSet& value )
 {
     KConfig* config = kapp->config();
     config->setGroup( group );
@@ -466,7 +447,195 @@ QString Settings::SettingsData::groupForDatabase( const QString& setting ) const
     return STR("%1 - %2").arg( setting ).arg( imageDirectory() );
 }
 
+void Settings::SettingsData::setCategorySyncingFields( const bool writing, const QString& category, const QValueList<Exif::Syncable::Kind>& fields )
+{
+    _setSyncing( writing, QString::fromAscii( "category_%1").arg( category ), fields );
+}
 
+QValueList<Exif::Syncable::Kind> Settings::SettingsData::categorySyncingFields( const bool writing, const QString& category ) const
+{
+    return _syncing( writing, QString::fromAscii("category_%1").arg( category ) );
+}
+
+void Settings::SettingsData::setCategorySyncingSuperGroups( const QString& category, const Exif::Syncable::SuperGroupHandling how )
+{
+    setValue( STR("MetadataSyncing"), STR("categorySyncingSuperGroups_%1").arg( category ), static_cast<int>( how ) );
+}
+
+Exif::Syncable::SuperGroupHandling Settings::SettingsData::categorySyncingSuperGroups( const QString& category ) const
+{
+    return static_cast<Exif::Syncable::SuperGroupHandling>( 
+            value( STR("MetadataSyncing"), STR("categorySyncingSuperGroups_%1").arg( category ), static_cast<int>(Exif::Syncable::Independent) ) );
+}
+
+void Settings::SettingsData::setCategorySyncingMultiValue(const QString& category, const Exif::Syncable::MultiValueHandling how )
+{
+    setValue( STR("MetadataSyncing"), STR("categorySyncingMultiValue_%1").arg( category ), static_cast<int>( how ) );
+}
+
+Exif::Syncable::MultiValueHandling Settings::SettingsData::categorySyncingMultiValue( const QString& category ) const
+{
+    return static_cast<Exif::Syncable::MultiValueHandling>( 
+            value( STR("MetadataSyncing"), STR("categorySyncingMultiValue_%1").arg( category ), static_cast<int>(Exif::Syncable::Repeat) ) );
+}
+
+void Settings::SettingsData::setCategorySyncingAddName( const QString& category, bool include )
+{
+    setValue( STR("MetadataSyncing"), STR("categorySyncingAddName_%1").arg( category ), include );
+}
+
+bool Settings::SettingsData::categorySyncingAddName( const QString& category )
+{
+    return value( STR("MetadataSyncing"), STR("categorySyncingAddName_%1").arg( category ), false );
+}
+
+void Settings::SettingsData::_setSyncing( const bool writing, const QString& identifier, const QValueList<Exif::Syncable::Kind>& fields )
+{
+    QString mode = writing ? QString::fromAscii("_W") : QString::fromAscii("_R");
+    QStringList list;
+    for (QValueList<Exif::Syncable::Kind>::const_iterator it = fields.begin(); it != fields.end(); ++it) {
+        list << QString::number( static_cast<int>( *it ) );
+    }
+    setValue( STR("MetadataSyncing"), STR("syncFields_%1").arg( identifier ) + mode, list.join( QString::fromAscii(";") ) );
+}
+
+QValueList<Exif::Syncable::Kind> Settings::SettingsData::_syncing( const bool writing, const QString& identifier ) const
+{
+    QString mode = writing ? QString::fromAscii("_W") : QString::fromAscii("_R");
+    QValueList<Exif::Syncable::Kind> list, allowedItems;
+    QStringList split = QStringList::split( QString::fromAscii(";"),
+            value( STR("MetadataSyncing"), STR("syncFields_%1").arg( identifier ) + mode, QString::null ) );
+    allowedItems = defaultFields( writing, identifier );
+
+    for (QStringList::const_iterator it = split.begin(); it != split.end(); ++it) {
+        Exif::Syncable::Kind item = static_cast<Exif::Syncable::Kind>( (*it).toInt() );
+        if ( allowedItems.contains( item ) )
+            list << item;
+    }
+
+    if (list.isEmpty())
+        list = allowedItems;
+
+    return list;
+}
+
+QValueList<Exif::Syncable::Kind> Settings::SettingsData::defaultFields( const bool writing, const QString& name ) const
+{
+    QValueList<Exif::Syncable::Kind> r, w;
+
+    if (name == QString::fromAscii("orientation")) {
+        r << Exif::Syncable::EXIF_ORIENTATION << Exif::Syncable::STOP;
+        w = r;
+    } else if ( name == QString::fromAscii("label")) {
+        r << Exif::Syncable::FILE_NAME <<
+            Exif::Syncable::STOP << Exif::Syncable::IPTC_HEADLINE <<
+            Exif::Syncable::EXIF_USER_COMMENT << Exif::Syncable::EXIF_DESCRIPTION <<
+            Exif::Syncable::JPEG_COMMENT << Exif::Syncable::EXIF_XPTITLE <<
+            Exif::Syncable::EXIF_XPSUBJECT << Exif::Syncable::IPTC_OBJECT_NAME <<
+            Exif::Syncable::IPTC_CAPTION;
+        w << Exif::Syncable::IPTC_HEADLINE <<
+            Exif::Syncable::STOP << Exif::Syncable::EXIF_USER_COMMENT <<
+            Exif::Syncable::EXIF_DESCRIPTION << Exif::Syncable::JPEG_COMMENT <<
+            Exif::Syncable::EXIF_XPTITLE << Exif::Syncable::EXIF_XPSUBJECT <<
+            Exif::Syncable::IPTC_CAPTION;
+    } else if ( name == QString::fromAscii("description")) {
+        r << Exif::Syncable::IPTC_CAPTION <<
+            Exif::Syncable::EXIF_USER_COMMENT << Exif::Syncable::EXIF_DESCRIPTION <<
+            Exif::Syncable::JPEG_COMMENT << Exif::Syncable::EXIF_XPCOMMENT <<
+            Exif::Syncable::EXIF_XPSUBJECT << Exif::Syncable::IPTC_OBJECT_NAME <<
+            Exif::Syncable::STOP << Exif::Syncable::IPTC_HEADLINE;
+        w << Exif::Syncable::IPTC_CAPTION <<
+            Exif::Syncable::STOP << Exif::Syncable::EXIF_USER_COMMENT <<
+            Exif::Syncable::EXIF_DESCRIPTION << Exif::Syncable::JPEG_COMMENT <<
+            Exif::Syncable::EXIF_XPCOMMENT << Exif::Syncable::EXIF_XPSUBJECT <<
+            Exif::Syncable::IPTC_HEADLINE;
+    } else if ( name == QString::fromAscii("date")) {
+        r << Exif::Syncable::EXIF_DATETIME << Exif::Syncable::EXIF_DATETIME_ORIGINAL <<
+            Exif::Syncable::EXIF_DATETIME_DIGITIZED << Exif::Syncable::FILE_MTIME <<
+            Exif::Syncable::FILE_CTIME << Exif::Syncable::STOP;
+        w << Exif::Syncable::EXIF_DATETIME << Exif::Syncable::STOP <<
+            /*Exif::Syncable::FILE_MTIME <<*/ Exif::Syncable::EXIF_DATETIME_ORIGINAL <<
+            Exif::Syncable::EXIF_DATETIME_DIGITIZED;
+    } else if ( name.startsWith( QString::fromAscii("category_") ) ) {
+        QString category = name.mid( QString::fromAscii("category_").length() );
+
+        // we need at least one random category for that vocabulary thingy...
+        DB::CategoryPtr someCategory = *( DB::ImageDB::instance()->categoryCollection()->categories().begin() );
+
+        if ( ( category == QString::fromLatin1("Keywords") ) ||
+                ( someCategory->standardCategories()[ QString::fromLatin1("Keywords") ] == category ) ) {
+            r << Exif::Syncable::STOP << Exif::Syncable::IPTC_KEYWORDS <<
+                Exif::Syncable::EXIF_XPKEYWORDS << Exif::Syncable::IPTC_SUPP_CAT;
+            w << Exif::Syncable::IPTC_KEYWORDS << Exif::Syncable::STOP << 
+                Exif::Syncable::EXIF_XPKEYWORDS << Exif::Syncable::IPTC_SUPP_CAT;
+        } else if ( ( category == QString::fromLatin1("Places") ) ||
+                ( someCategory->standardCategories()[ QString::fromLatin1("Places") ] == category ) ) {
+            r << Exif::Syncable::STOP <<
+                Exif::Syncable::IPTC_LOCATION_CODE << Exif::Syncable::IPTC_LOCATION_NAME <<
+                Exif::Syncable::IPTC_CITY << Exif::Syncable::IPTC_SUB_LOCATION <<
+                Exif::Syncable::IPTC_PROVINCE_STATE << Exif::Syncable::IPTC_COUNTRY_NAME <<
+                Exif::Syncable::IPTC_COUNTRY_CODE << Exif::Syncable::IPTC_SUPP_CAT;
+            w << Exif::Syncable::IPTC_LOCATION_NAME << Exif::Syncable::STOP <<
+                Exif::Syncable::IPTC_LOCATION_CODE << Exif::Syncable::IPTC_CITY << Exif::Syncable::IPTC_SUB_LOCATION <<
+                Exif::Syncable::IPTC_PROVINCE_STATE << Exif::Syncable::IPTC_COUNTRY_NAME <<
+                Exif::Syncable::IPTC_COUNTRY_CODE << Exif::Syncable::IPTC_SUPP_CAT;
+        } else {
+            r << Exif::Syncable::STOP << Exif::Syncable::EXIF_DESCRIPTION <<
+                Exif::Syncable::EXIF_USER_COMMENT << Exif::Syncable::EXIF_XPTITLE <<
+                Exif::Syncable::EXIF_XPCOMMENT << Exif::Syncable::EXIF_XPKEYWORDS <<
+                Exif::Syncable::EXIF_XPSUBJECT << Exif::Syncable::IPTC_HEADLINE <<
+                Exif::Syncable::IPTC_CAPTION << Exif::Syncable::IPTC_OBJECT_NAME <<
+                Exif::Syncable::IPTC_SUBJECT << Exif::Syncable::IPTC_SUPP_CAT <<
+                Exif::Syncable::IPTC_KEYWORDS << Exif::Syncable::IPTC_LOCATION_CODE <<
+                Exif::Syncable::IPTC_LOCATION_NAME << Exif::Syncable::IPTC_CITY <<
+                Exif::Syncable::IPTC_SUB_LOCATION << Exif::Syncable::IPTC_PROVINCE_STATE <<
+                Exif::Syncable::IPTC_COUNTRY_CODE << Exif::Syncable::IPTC_COUNTRY_NAME;
+            w = r;
+        }
+    }
+
+    return writing ? w : r;
+}
+
+void Settings::SettingsData::setLabelSyncing( const bool writing, const QValueList<Exif::Syncable::Kind>& fields )
+{
+    _setSyncing( writing, QString::fromAscii("label"), fields );
+}
+
+QValueList<Exif::Syncable::Kind> Settings::SettingsData::labelSyncing( const bool writing ) const
+{
+    return _syncing( writing, QString::fromAscii("label") );
+}
+
+void Settings::SettingsData::setDescriptionSyncing( const bool writing, const QValueList<Exif::Syncable::Kind>& fields )
+{
+    _setSyncing( writing, QString::fromAscii("description"), fields );
+}
+
+QValueList<Exif::Syncable::Kind> Settings::SettingsData::descriptionSyncing( const bool writing ) const
+{
+    return _syncing( writing, QString::fromAscii("description") );
+}
+
+void Settings::SettingsData::setOrientationSyncing( const bool writing, const QValueList<Exif::Syncable::Kind>& fields )
+{
+    _setSyncing( writing, QString::fromAscii("orientation"), fields );
+}
+
+QValueList<Exif::Syncable::Kind> Settings::SettingsData::orientationSyncing( const bool writing ) const
+{
+    return _syncing( writing, QString::fromAscii("orientation") );
+}
+
+void Settings::SettingsData::setDateSyncing( const bool writing, const QValueList<Exif::Syncable::Kind>& fields )
+{
+    _setSyncing( writing, QString::fromAscii("date"), fields );
+}
+
+QValueList<Exif::Syncable::Kind> Settings::SettingsData::dateSyncing( const bool writing ) const
+{
+    return _syncing( writing, QString::fromAscii("date") );
+}
 
 void Settings::SettingsData::setThumbnailCache( int value )
 {
