@@ -46,6 +46,10 @@
 #include "ViewerWidget.h"
 #include <qapplication.h>
 #include <qeventloop.h>
+#include <qfileinfo.h>
+#include <TextDisplay.h>
+#include <kprocess.h>
+#include <kstandarddirs.h>
 
 #ifdef HASEXIV2
 #  include "Exif/InfoDialog.h"
@@ -60,7 +64,7 @@
  * The class consists of these components:
  * <ul>
  * <li>\ref ViewerWidget - This is the topmost widget used as the viewer.
- * <li>\ref Display, \ref ImageDisplay, and \ref VideoDisplay - Widgets hierarchy which takes care of the actual displaying of content.
+ * <li>\ref Display, \ref ImageDisplay, \ref VideoDisplay and \ref TextDisplay - Widgets hierarchy which takes care of the actual displaying of content.
  * <li> \ref DisplayAreaHandler, \ref DrawHandler, and \ref ViewHandler - Class Hierarchy which interprets mouse gestures.
  * <li> \ref Draw, \ref LineDraw, \ref CircleDraw, and \ref RectDraw - Class Hierarchy containing information about drawing on images.
  * <li> \ref InfoBox - Widget implementing the informatiom box
@@ -88,10 +92,13 @@ Viewer::ViewerWidget::ViewerWidget( const char* name )
     _stack = new QWidgetStack( this, "stack" );
 
     _display = _imageDisplay = new ImageDisplay( _stack ); // Must be created before the toolbar.
+    _textDisplay = new TextDisplay( _stack );
+    _stack->addWidget( _textDisplay );
     _videoDisplay = new VideoDisplay( _stack );
     connect( _videoDisplay, SIGNAL( stopped() ), this, SLOT( videoStopped() ) );
 
     connect( _imageDisplay, SIGNAL( possibleChange() ), this, SLOT( updateCategoryConfig() ) );
+    connect( _imageDisplay, SIGNAL( imageReady() ), this, SLOT( updateInfoBox() ) );
     connect( _imageDisplay, SIGNAL( setCaptionInfo(const QString&) ),
              this, SLOT( setCaptionWithDetail(const QString&) ) );
     createToolBar();
@@ -104,7 +111,7 @@ Viewer::ViewerWidget::ViewerWidget( const char* name )
     // This must not be added to the layout, as it is standing on top of
     // the ImageDisplay
     _infoBox = new InfoBox( this );
-    _infoBox->setShown( Settings::SettingsData::instance()->showInfoBox() );
+    _infoBox->hide();
 
     setupContextMenu();
 
@@ -114,8 +121,16 @@ Viewer::ViewerWidget::ViewerWidget( const char* name )
     _speedDisplay = new SpeedDisplay( this );
 
     setFocusPolicy( StrongFocus );
-}
 
+    const QString xdgScreenSaver = KStandardDirs::findExe( QString::fromAscii("xdg-screensaver") );
+    if ( !xdgScreenSaver.isEmpty() ) {
+        KProcess proc;
+        proc << xdgScreenSaver;
+        proc << "suspend";
+        proc << QString::number( winId() );
+        proc.start( KProcess::DontCare );
+    }
+}
 
 void Viewer::ViewerWidget::setupContextMenu()
 {
@@ -170,6 +185,11 @@ void Viewer::ViewerWidget::createShowContextMenu()
     taction->plug( showPopup );
     taction->setChecked( Settings::SettingsData::instance()->showDrawings() );
 
+    taction = new KToggleAction( i18n("Show Label"), 0, _actions, "viewer-show-label" );
+    connect( taction, SIGNAL( toggled( bool ) ), this, SLOT( toggleShowLabel( bool ) ) );
+    taction->plug( showPopup );
+    taction->setChecked( Settings::SettingsData::instance()->showLabel() );
+
     taction = new KToggleAction( i18n("Show Description"), 0, _actions, "viewer-show-description" );
     connect( taction, SIGNAL( toggled( bool ) ), this, SLOT( toggleShowDescription( bool ) ) );
     taction->plug( showPopup );
@@ -254,13 +274,13 @@ void Viewer::ViewerWidget::createRotateMenu()
 {
     _rotateMenu = new QPopupMenu( _popup );
 
-    KAction* action = new KAction( i18n("Rotate 90 Degrees"), Key_9, this, SLOT( rotate90() ), _actions, "viewer-rotate90" );
+    KAction* action = new KAction( i18n("Rotate counterclockwise"), Key_9, this, SLOT( rotate90() ), _actions, "viewer-rotate90" );
     action->plug( _rotateMenu );
 
-    action = new KAction( i18n("Rotate 180 Degrees"), Key_8, this, SLOT( rotate180() ), _actions, "viewer-rotate180" );
+    action = new KAction( i18n("Flip Over"), Key_8, this, SLOT( rotate180() ), _actions, "viewer-rotate180" );
     action->plug( _rotateMenu );
 
-    action = new KAction( i18n("Rotate 270 Degrees"), Key_7, this, SLOT( rotate270() ), _actions, "viewer-rotare270" );
+    action = new KAction( i18n("Rotate clockwise"), Key_7, this, SLOT( rotate270() ), _actions, "viewer-rotare270" );
     action->plug( _rotateMenu );
 
     _popup->insertItem( QIconSet(), i18n("Rotate"), _rotateMenu );
@@ -361,14 +381,7 @@ void Viewer::ViewerWidget::createSlideShowMenu()
 void Viewer::ViewerWidget::load( const QStringList& list, int index )
 {
     _list = list;
-
-    QStringList images;
-    for( QStringList::ConstIterator it = list.begin(); it != list.end(); ++it ) {
-        if ( !Utilities::isVideo( *it ) )
-            images << *it;
-    }
     _imageDisplay->setImageList( list );
-
     _current = index;
     load();
 
@@ -380,11 +393,19 @@ void Viewer::ViewerWidget::load( const QStringList& list, int index )
 
 void Viewer::ViewerWidget::load()
 {
-    bool isVideo = Utilities::isVideo( currentInfo()->fileName() );
-    if ( isVideo )
-        _display = _videoDisplay;
-    else
-        _display = _imageDisplay;
+    bool isReadable = QFileInfo( currentInfo()->fileName() ).isReadable();
+    bool isVideo = isReadable && Utilities::isVideo( currentInfo()->fileName() );
+
+    if ( isReadable ) {
+        if ( isVideo )
+            _display = _videoDisplay;
+        else
+            _display = _imageDisplay;
+    } else {
+        _display = _textDisplay;
+        _textDisplay->setText( i18n("File not available") );
+        updateInfoBox();
+    }
 
     _stack->raiseWidget( _display );
 
@@ -417,7 +438,6 @@ void Viewer::ViewerWidget::load()
     }
 
     setCaptionWithDetail( QString() );
-    updateInfoBox();
 
     // PENDING(blackie) This needs to be improved, so that it shows the actions only if there are that many images to jump.
     for( QPtrList<KAction>::const_iterator it = _forwardActions.begin(); it != _forwardActions.end(); ++it )
@@ -429,6 +449,9 @@ void Viewer::ViewerWidget::load()
 
     if ( _isRunningSlideShow )
         _slideShowTimer->changeInterval( _slideShowPause );
+
+    if ( _display == _textDisplay )
+        updateInfoBox();
 }
 
 void Viewer::ViewerWidget::setCaptionWithDetail( const QString& detail ) {
@@ -445,6 +468,9 @@ void Viewer::ViewerWidget::contextMenuEvent( QContextMenuEvent * e )
 
 void Viewer::ViewerWidget::showNextN(int n)
 {
+    if ( _display == _videoDisplay )
+        _videoDisplay->stop();
+
     save();
     if ( _current +1 < (int) _list.count() )  {
         _current += n;
@@ -477,6 +503,9 @@ void Viewer::ViewerWidget::showNext1000()
 
 void Viewer::ViewerWidget::showPrevN(int n)
 {
+    if ( _display == _videoDisplay )
+        _videoDisplay->stop();
+
     save();
     if ( _current > 0  )  {
         _current -= n;
@@ -532,6 +561,12 @@ void Viewer::ViewerWidget::toggleShowInfoBox( bool b )
     updateInfoBox();
 }
 
+void Viewer::ViewerWidget::toggleShowLabel( bool b )
+{
+    Settings::SettingsData::instance()->setShowLabel( b );
+    updateInfoBox();
+}
+
 void Viewer::ViewerWidget::toggleShowDescription( bool b )
 {
     Settings::SettingsData::instance()->setShowDescription( b );
@@ -656,7 +691,7 @@ bool Viewer::ViewerWidget::close( bool alsoDelete)
     return QWidget::close( alsoDelete );
 }
 
-DB::ImageInfoPtr Viewer::ViewerWidget::currentInfo()
+DB::ImageInfoPtr Viewer::ViewerWidget::currentInfo() const
 {
     return DB::ImageDB::instance()->info(_list[ _current]); // PENDING(blackie) can we postpone this lookup?
 }
@@ -758,6 +793,17 @@ void Viewer::ViewerWidget::updateInfoBox()
 
 Viewer::ViewerWidget::~ViewerWidget()
 {
+    const QString xdgScreenSaver = KStandardDirs::findExe( QString::fromAscii("xdg-screensaver") );
+    if ( !xdgScreenSaver.isEmpty() ) {
+        KProcess proc;
+        proc << xdgScreenSaver;
+        proc << "resume";
+        proc << QString::number( winId() );
+        // if we don't wait here, xdg-screensaver realizes that the window is
+        // already gone and doesn't re-activate the screensaver
+        proc.start( KProcess::Block );
+    }
+
     if ( _latest == this )
         _latest = 0;
 }
@@ -806,11 +852,13 @@ void Viewer::ViewerWidget::slotStartStopSlideShow()
     _isRunningSlideShow = !_isRunningSlideShow && _list.count() != 1;
 
     if ( wasRunningSlideShow ) {
+        _startStopSlideShow->setText( i18n("Run Slideshow") );
         _slideShowTimer->stop();
         if ( _list.count() != 1 )
             _speedDisplay->end();
     }
     else {
+        _startStopSlideShow->setText( i18n("Stop Slideshow") );
         if ( currentInfo()->mediaType() != DB::Video )
             _slideShowTimer->start( _slideShowPause, true );
         _speedDisplay->start();
